@@ -18,20 +18,21 @@ use IO::Socket;
 use Net::SSLeay 1.21;
 use Exporter ();
 use Errno qw( EAGAIN ETIMEDOUT );
+use Socket 1.95 qw( inet_pton );
 use Carp;
 use strict;
 
-use constant {
-	SSL_VERIFY_NONE => Net::SSLeay::VERIFY_NONE(),
-	SSL_VERIFY_PEER => Net::SSLeay::VERIFY_PEER(),
-	SSL_VERIFY_FAIL_IF_NO_PEER_CERT => Net::SSLeay::VERIFY_FAIL_IF_NO_PEER_CERT(),
-	SSL_VERIFY_CLIENT_ONCE => Net::SSLeay::VERIFY_CLIENT_ONCE(),
-	# from openssl/ssl.h, should be better in Net::SSLeay
-	SSL_SENT_SHUTDOWN => 1,
-	SSL_RECEIVED_SHUTDOWN => 2,
-};
+use constant SSL_VERIFY_NONE => Net::SSLeay::VERIFY_NONE();
+use constant SSL_VERIFY_PEER => Net::SSLeay::VERIFY_PEER();
+use constant SSL_VERIFY_FAIL_IF_NO_PEER_CERT => Net::SSLeay::VERIFY_FAIL_IF_NO_PEER_CERT();
+use constant SSL_VERIFY_CLIENT_ONCE => Net::SSLeay::VERIFY_CLIENT_ONCE();
 
+# from openssl/ssl.h; should be better in Net::SSLeay
+use constant SSL_SENT_SHUTDOWN => 1;
+use constant SSL_RECEIVED_SHUTDOWN => 2;
 
+use constant DEFAULT_CIPHER_LIST => 'ALL:!LOW';
+use constant DEFAULT_VERSION     => 'SSLv23:!SSLv2';
 
 # non-XS Versions of Scalar::Util will fail
 BEGIN{
@@ -61,24 +62,29 @@ use vars qw(@ISA $VERSION $DEBUG $SSL_ERROR $GLOBAL_CONTEXT_ARGS @EXPORT );
 }
 
 my @caller_force_inet4; # in case inet4 gets forced we store here who forced it
-my $can_ipv6;           # true if we successfully enabled ipv6 while loading
 
 BEGIN {
 	# Declare @ISA, $VERSION, $GLOBAL_CONTEXT_ARGS
 
+	# if we have IO::Socket::IP >= 0.11 we will use this in preference
+	# because it can handle both IPv4 and IPv6
+	if ( eval { require IO::Socket::IP; IO::Socket::IP->VERSION(0.11); } ) {
+		@ISA = qw(IO::Socket::IP);
+		constant->import( CAN_IPV6 => "IO::Socket::IP" );
+	}
 	# if we have IO::Socket::INET6 we will use this not IO::Socket::INET, because
 	# it can handle both IPv4 and IPv6. If we don't have INET6 available fall back
 	# to INET
-	if ( ! eval {
-		require Socket6;
-		Socket6->import( 'inet_pton' );
-		require IO::Socket::INET6;
+	elsif( eval { require IO::Socket::INET6; } ) {
 		@ISA = qw(IO::Socket::INET6);
-		$can_ipv6 = 1;
-	}) {
-		@ISA = qw(IO::Socket::INET);
+		constant->import( CAN_IPV6 => "IO::Socket::INET6" );
 	}
-	$VERSION = '1.43_1';
+	else {
+		@ISA = qw(IO::Socket::INET);
+		constant->import( CAN_IPV6 => '' );
+	}
+
+	$VERSION = '1.74_1';
 	$GLOBAL_CONTEXT_ARGS = {};
 
 	#Make $DEBUG another name for $Net::SSLeay::trace
@@ -151,11 +157,11 @@ sub import {
 				# either we don't support it or we disabled it by explicitly
 				# loading it with 'inet4'. In this case re-enable but warn
 				# because this is probably an error
-				if ( $can_ipv6 ) {
-					@ISA = 'IO::Socket::INET6';
+				if ( CAN_IPV6 ) {
+					@ISA = ( CAN_IPV6 );
 					warn "IPv6 support re-enabled in __PACKAGE__, got disabled in file $caller_force_inet4[1] line $caller_force_inet4[2]";
 				} else {
-					die "INET6 is not supported, missing Socket6 or IO::Socket::INET6";
+					die "INET6 is not supported, install IO::Socket::INET6";
 				}
 			}
 		} elsif ( /^:?debug(\d+)/ ) {
@@ -169,6 +175,9 @@ sub import {
 	goto &Exporter::import;
 }
 
+my %CREATED_IN_THIS_THREAD;
+sub CLONE { %CREATED_IN_THIS_THREAD = (); }
+
 # You might be expecting to find a new() subroutine here, but that is
 # not how IO::Socket::INET works.  All configuration gets performed in
 # the calls to configure() and either connect() or accept().
@@ -182,7 +191,7 @@ sub configure {
 	# work around Bug in IO::Socket::INET6 where it doesn't use the
 	# right family for the socket on BSD systems:
 	# http://rt.cpan.org/Ticket/Display.html?id=39550
-	if ( $can_ipv6 && ! $arg_hash->{Domain} &&
+	if ( CAN_IPV6 eq "IO::Socket::INET6" && ! $arg_hash->{Domain} &&
 		! ( $arg_hash->{LocalAddr} || $arg_hash->{LocalHost} ) &&
 		(my $peer = $arg_hash->{PeerAddr} || $arg_hash->{PeerHost})) {
 		# set Domain to AF_INET/AF_INET6 if there is only one choice
@@ -226,11 +235,13 @@ sub configure_SSL {
 		SSL_server => $is_server,
 		SSL_use_cert => $is_server,
 		SSL_check_crl => 0,
-		SSL_version	=> 'sslv23',
+		SSL_version	=> DEFAULT_VERSION,
 		SSL_verify_mode => SSL_VERIFY_NONE,
 		SSL_verify_callback => undef,
 		SSL_verifycn_scheme => undef,  # don't verify cn
 		SSL_verifycn_name => undef,    # use from PeerAddr/PeerHost
+		SSL_npn_protocols => undef,    # meaning depends whether on server or client side
+		SSL_honor_cipher_order => 0,   # client order gets preference
 	);
 
 	# common problem forgetting SSL_use_cert
@@ -287,7 +298,7 @@ sub configure_SSL {
 		$arg_hash->{SSL_verify_callback} = sub {
 			my ($ok,$ctx_store,$certname,$error,$cert) = @_;
 			$ok = $vcb->($ok,$ctx_store,$certname,$error,$cert) if $vcb;
-			$ok or return;
+			$ok or return 0;
 			my $depth = Net::SSLeay::X509_STORE_CTX_get_error_depth($ctx_store);
 			return $ok if $depth != 0;
 
@@ -334,6 +345,11 @@ sub connect {
 		DEBUG(2, 'socket not yet connected' );
 		$self->SUPER::connect(@_) || return;
 		DEBUG(2,'socket connected' );
+
+		# IO::Socket works around systems, which return EISCONN or similar
+		# on non-blocking re-connect by returning true, even if $! is set
+		# but it does not clear $!, so do it here
+		$! = undef;
 	}
 	return $self->connect_SSL;
 }
@@ -356,15 +372,35 @@ sub connect_SSL {
 		$ctx = ${*$self}{'_SSL_ctx'};  # Reference to real context
 		$ssl = ${*$self}{'_SSL_object'} = Net::SSLeay::new($ctx->{context})
 			|| return $self->error("SSL structure creation failed");
+		$CREATED_IN_THIS_THREAD{$ssl} = 1;
 
 		Net::SSLeay::set_fd($ssl, $fileno)
 			|| return $self->error("SSL filehandle association failed");
 
-		if ( my $cl = $arg_hash->{SSL_cipher_list} ) {
+		if ( my $cl = exists $arg_hash->{SSL_cipher_list} 
+		    ? $arg_hash->{SSL_cipher_list} 
+		    :  DEFAULT_CIPHER_LIST ) {
 			Net::SSLeay::set_cipher_list($ssl, $cl )
 				|| return $self->error("Failed to set SSL cipher list");
 		}
 
+		if (Net::SSLeay::OPENSSL_VERSION_NUMBER() >= 0x009080ef) {
+			my $host;
+			if ( exists $arg_hash->{SSL_hostname} ) {
+				# explicitly given
+				# can be set to undef/'' to not use extension
+				$host = $arg_hash->{SSL_hostname}
+			} elsif ( $host = $arg_hash->{PeerAddr} || $arg_hash->{PeerHost} ) {
+				# implicitly given
+				$host =~s{:[a-zA-Z0-9_\-]+$}{};
+				# should be hostname, not IPv4/6
+				$host = undef if $host !~m{[a-z_]} or $host =~m{:};
+			}
+			# define SSL_CTRL_SET_TLSEXT_HOSTNAME 55
+			# define TLSEXT_NAMETYPE_host_name 0
+			Net::SSLeay::ctrl($ssl,55,0,$host) if $host;
+		}
+
 		$arg_hash->{PeerAddr} || $self->_update_peer;
 		my $session = $ctx->session_cache( $arg_hash->{PeerAddr}, $arg_hash->{PeerPort} );
 		Net::SSLeay::set_session($ssl, $session) if ($session);
@@ -515,11 +551,14 @@ sub accept_SSL {
 
 		$ssl = ${*$socket}{'_SSL_object'} = Net::SSLeay::new($ctx->{context})
 			|| return $socket->error("SSL structure creation failed");
+		$CREATED_IN_THIS_THREAD{$ssl} = 1;
 
 		Net::SSLeay::set_fd($ssl, $fileno)
 			|| return $socket->error("SSL filehandle association failed");
 
-		if ( my $cl = $arg_hash->{SSL_cipher_list} ) {
+		if ( my $cl = exists $arg_hash->{SSL_cipher_list} 
+		    ? $arg_hash->{SSL_cipher_list}
+		    : DEFAULT_CIPHER_LIST) {
 			Net::SSLeay::set_cipher_list($ssl, $cl )
 				|| return $socket->error("Failed to set SSL cipher list");
 		}
@@ -718,14 +757,24 @@ sub getc {
 
 sub readline {
 	my $self = shift;
-	my $ssl = $self->_get_ssl_object || return;
 
-	if (wantarray) {
-		my ($buf,$err) = Net::SSLeay::ssl_read_all($ssl);
-		return $self->error( "SSL read error" ) if $err;
-		if ( !defined($/) ) {
-			return $buf;
-		} elsif ( ref($/) ) {
+	if ( not defined $/ or wantarray) {
+		# read all and split
+
+		my $buf = '';
+		while (1) {
+			my $rv = $self->sysread($buf,2**16,length($buf));
+			if ( ! defined $rv ) {
+				next if $!{EINTR};
+				return;
+			} elsif ( ! $rv ) {
+				last
+			}
+		}
+
+		if ( ! defined $/ ) {
+			return $buf
+		} elsif ( ref($/)) {
 			my $size = ${$/};
 			die "bad value in ref \$/: $size" unless $size>0;
 			return $buf=~m{\G(.{1,$size})}g;
@@ -736,59 +785,100 @@ sub readline {
 		}
 	}
 
-	if ( !defined($/) ) {
-		my ($buf,$err) = Net::SSLeay::ssl_read_all($ssl);
-		return $self->error( "SSL read error" ) if $err;
-		return $buf;
-	} elsif ( ref($/) ) {
+	# read only one line
+	if ( ref($/) ) {
 		my $size = ${$/};
+		# read record of $size bytes
 		die "bad value in ref \$/: $size" unless $size>0;
-		my ($buf,$err) = Net::SSLeay::ssl_read_all($ssl,$size);
-		return $self->error( "SSL read error" ) if $err;
-		return $buf;
-	} elsif ( $/ ne '' ) {
-		my $line = Net::SSLeay::ssl_read_until($ssl,$/);
-		return $self->error( "SSL read error" ) if $line eq '';
-		return $line;
-	} else {
-		# $/ is ''
-		# ^.*?\n\n+, need peek to find all \n at the end
-		die "empty \$/ is not supported if I don't have peek"
-			if Net::SSLeay::OPENSSL_VERSION_NUMBER() < 0x0090601f;
-
-		# find first occurence of \n\n
 		my $buf = '';
-		my $eon = 0;
-		while (1) {
-			defined( Net::SSLeay::peek($ssl,1)) || last; # peek more, can block
-			my $pending = Net::SSLeay::pending($ssl);
-			$buf .= Net::SSLeay::peek( $ssl,$pending );	 # will not block
-			if ( !$eon ) {
-				my $pos = index( $buf,"\n\n");
-				next if $pos<0; # newlines not found
-				$eon = $pos+2;	# pos after second newline
+		while ( $size>length($buf)) {
+			my $rv = $self->sysread($buf,$size-length($buf),length($buf));
+			if ( ! defined $rv ) {
+				next if $!{EINTR};
+				return;
+			} elsif ( ! $rv ) {
+				last
 			}
-			# $eon >= 2	 == bytes incl last known \n
-			while ( index( $buf,"\n",$eon ) == $eon ) {
-				# the next char ist \n too
-				$eon++;
-			}
-			last if $eon < length($buf); # found last \n before end of buf
 		}
-		if ( $eon > 0 ) {
-			# found something
-			# readed peeked data until $eon from $ssl
-			return Net::SSLeay::ssl_read_all( $ssl,$eon );
-		} else {
-			# found nothing
-			# return all what we have
-			if ( my $l = length($buf)) {
-				return Net::SSLeay::ssl_read_all( $ssl,$l );
-			} else {
-				return $self->error( "SSL read error" );
+		return $buf;
+	}
+
+	my ($delim0,$delim1) = $/ eq '' ? ("\n\n","\n"):($/,'');
+
+	if ( Net::SSLeay::OPENSSL_VERSION_NUMBER() < 0x0090601f ) {
+		# no usable peek - need to read byte after byte
+		die "empty \$/ is not supported if I don't have peek" if $delim1 ne '';
+		my $buf = '';
+		while (1) {
+			my $rv = $self->sysread($buf,1,length($buf));
+			if ( ! defined $rv ) {
+				next if $!{EINTR};
+				return;
+			} elsif ( ! $rv ) {
+				last
 			}
+			index($buf,$delim0) >= 0 and last;
+		}
+		return $buf;
+	}
+
+
+	# find first occurence of $delim0 followed by as much as possible $delim1
+	my $buf = '';
+	my $eod = 0;  # pointer into $buf after $delim0 $delim1*
+	my $ssl = $self->_get_ssl_object or return;
+	while (1) {
+
+		# block until we have more data or eof
+		my $poke = Net::SSLeay::peek($ssl,1);
+		if ( ! defined $poke or $poke eq '' ) {
+			next if $!{EINTR};
+		}
+
+		my $skip = 0;
+
+		# peek into available data w/o reading
+		my $pending = Net::SSLeay::pending($ssl);
+		if ( $pending and 
+			( my $pb = Net::SSLeay::peek( $ssl,$pending )) ne '' ) {
+			$buf .= $pb
+		} else {
+			return $buf eq '' ? ():$buf;
+		};
+		if ( !$eod ) {
+			my $pos = index( $buf,$delim0 );
+			if ( $pos<0 ) {
+				$skip = $pending
+			} else {
+				$eod = $pos + length($delim0); # pos after delim0
+			}
+		}
+
+		if ( $eod ) {
+			if ( $delim1 ne '' ) {
+				# delim0 found, check for as much delim1 as possible
+				while ( index( $buf,$delim1,$eod ) == $eod ) {
+					$eod+= length($delim1);
+				}
+			}
+			$skip = $pending - ( length($buf) - $eod );
+		}
+
+		# remove data from $self which I already have in buf
+		while ( $skip>0 ) {
+			if ($self->sysread(my $p,$skip,0)) {
+				$skip -= length($p);
+				next;
+			}
+			$!{EINTR} or last;
+		}
+
+		if ( $eod and ( $delim1 eq '' or $eod < length($buf))) {
+			# delim0 found and there can be no more delim1 pending
+			last
 		}
 	}
+	return substr($buf,0,$eod);
 }
 
 sub close {
@@ -1086,6 +1176,8 @@ sub dump_peer_certificate {
 			$scheme = $scheme{$scheme} or croak "scheme $scheme not defined";
 		}
 
+		return 1 if ! %$scheme; # 'none'
+
 		# get data from certificate
 		my $commonName = $dispatcher{cn}->($cert);
 		my @altNames = $dispatcher{subjectAltNames}->($cert);
@@ -1101,10 +1193,8 @@ sub dump_peer_certificate {
 		my $ipn;
 		if ( $identity =~m{:} ) {
 			# no IPv4 or hostname have ':'	in it, try IPv6.
-			#  make sure that Socket6 was loaded properly
-			UNIVERSAL::can( __PACKAGE__, 'inet_pton' ) or croak
-				q[Looks like IPv6 address, make sure that Socket6 is loaded or make "use IO::Socket::SSL 'inet6'];
-			$ipn = inet_pton( $identity ) or croak "'$identity' is not IPv6, but neither IPv4 nor hostname";
+			$ipn = inet_pton(AF_INET6,$identity) 
+				or croak "'$identity' is not IPv6, but neither IPv4 nor hostname";
 		} elsif ( $identity =~m{^\d+\.\d+\.\d+\.\d+$} ) {
 			 # definitly no hostname, try IPv4
 			$ipn = inet_aton( $identity ) or croak "'$identity' is not IPv4, but neither IPv6 nor hostname";
@@ -1208,7 +1298,7 @@ sub get_ssleay_error {
 
 sub error {
 	my ($self, $error, $destroy_socket) = @_;
-	$error .= Net::SSLeay::ERR_error_string(Net::SSLeay::ERR_get_error());
+	$error .= ' '.Net::SSLeay::ERR_error_string(Net::SSLeay::ERR_get_error());
 	DEBUG(2, $error."\n".$self->get_ssleay_error());
 	$SSL_ERROR = dualvar( -1, $error );
 	${*$self}{'_SSL_last_err'} = $SSL_ERROR if (ref($self));
@@ -1217,10 +1307,13 @@ sub error {
 
 
 sub DESTROY {
-	my $self = shift || return;
-	$self->close(_SSL_in_DESTROY => 1, SSL_no_shutdown => 1)
-		if ${*$self}{'_SSL_opened'};
-	delete(${*$self}{'_SSL_ctx'});
+	my $self = shift or return;
+	my $ssl = ${*$self}{_SSL_object} or return;
+	if ($CREATED_IN_THIS_THREAD{$ssl}) {
+		$self->close(_SSL_in_DESTROY => 1, SSL_no_shutdown => 1)
+			if ${*$self}{'_SSL_opened'};
+		delete(${*$self}{'_SSL_ctx'});
+	}
 }
 
 
@@ -1253,6 +1346,13 @@ sub set_ctx_defaults {
 	}
 }
 
+sub next_proto_negotiated {
+	my $self = shift;
+	return $self->error("NPN not supported in Net::SSLeay")
+	    if ! exists &Net::SSLeay::P_next_proto_negotiated;
+	my $ssl = $self->_get_ssl_object || return;
+	return Net::SSLeay::P_next_proto_negotiated($ssl);
+}
 
 sub opened {
 	my $self = shift;
@@ -1358,17 +1458,49 @@ sub new {
 		return $ctx_object if ($ctx_object = ${*$ctx_object}{'_SSL_ctx'});
 	}
 
-	my $ctx;
-	foreach ($arg_hash->{'SSL_version'}) {
-		$ctx = /^sslv2$/i ? Net::SSLeay::CTX_v2_new()	 :
-			   /^sslv3$/i ? Net::SSLeay::CTX_v3_new()	 :
-			   /^tlsv1$/i ? Net::SSLeay::CTX_tlsv1_new() :
-							Net::SSLeay::CTX_new();
+	my $ver;
+	my $disable_ver = 0;
+	for (split(/\s*:\s*/,$arg_hash->{SSL_version})) {
+	    m{^(!?)(?:(SSL(?:v2|v3|v23|v2/3))|(TLSv1))$}i 
+		or croak("invalid SSL_version specified");
+	    my $not = $1;
+	    ( my $v = lc($2||$3) ) =~s{^(...)}{\U$1};
+	    $v =~s{/}{}; # interpret SSLv2/3 as SSLv23
+	    if ( $not ) {
+		$disable_ver |= 
+		    $v eq 'SSLv2' ? 0x01000000 : # SSL_OP_NO_SSLv2 
+		    $v eq 'SSLv3' ? 0x02000000 : # SSL_OP_NO_SSLv3 
+		    $v eq 'TLSv1' ? 0x04000000 : # SSL_OP_NO_TLSv1
+		    croak("cannot disable version $_");
+	    } else {
+		croak("cannot set multiple SSL protocols in SSL_version")
+		    if $ver && $v ne $ver;
+		$ver = $v;
+	    }
 	}
 
-	$ctx || return IO::Socket::SSL->error("SSL Context init failed");
+	my $sub =  UNIVERSAL::can( 'Net::SSLeay',
+	    $ver eq 'SSLv2' ? 'CTX_v2_new' :
+	    $ver eq 'SSLv3' ? 'CTX_v3_new' :
+	    $ver eq 'TLSv1' ? 'CTX_tlsv1_new' :
+	    'CTX_new'
+	) or return IO::Socket::SSL->error("SSL Version $ver not supported");
+	my $ctx = $sub->() or return 
+	    IO::Socket::SSL->error("SSL Context init failed");
 
-	Net::SSLeay::CTX_set_options($ctx, Net::SSLeay::OP_ALL());
+	Net::SSLeay::CTX_set_options($ctx, Net::SSLeay::OP_ALL() | $disable_ver );
+	if ( $arg_hash->{SSL_honor_cipher_order} ) {
+	    Net::SSLeay::CTX_set_options($ctx, 0x00400000);
+	}
+
+	# if we don't set session_id_context if client certicate is expected
+	# client session caching will fail
+	# if user does not provide explicit id just use the stringification
+	# of the context
+	if ( my $id = $arg_hash->{SSL_session_id_context} 
+	    || ( $arg_hash->{SSL_verify_mode} & 0x01 ) && "$ctx" ) {
+	    Net::SSLeay::CTX_set_session_id_context($ctx,$id,length($id));
+	}
 
 	# SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER makes syswrite return if at least one
 	# buffer was written and not block for the rest
@@ -1377,6 +1509,18 @@ sub new {
 	Net::SSLeay::CTX_set_mode( $ctx,
 		SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER|SSL_MODE_ENABLE_PARTIAL_WRITE);
 
+	if ( my $proto_list = $arg_hash->{SSL_npn_protocols} ) {
+		return IO::Socket::SSL->error("NPN not supported in Net::SSLeay")
+		    if ! exists &Net::SSLeay::P_next_proto_negotiated;
+		if($arg_hash->{SSL_server}) {
+			# on server side SSL_npn_protocols means a list of advertised protocols
+			Net::SSLeay::CTX_set_next_protos_advertised_cb($ctx, $proto_list);
+		} else {
+			# on client side SSL_npn_protocols means a list of prefered protocols
+			# negotiation algorithm used is "as-openssl-implements-it"
+			Net::SSLeay::CTX_set_next_proto_select_cb($ctx, $proto_list);
+		}
+	}
 
 	my $verify_mode = $arg_hash->{SSL_verify_mode};
 	if ( $verify_mode != Net::SSLeay::VERIFY_NONE() and
@@ -1646,20 +1790,50 @@ that came bundled with IO::Socket::INET, plus (optionally) the ones that follow:
 
 =over 2
 
+=item SSL_hostname
+
+This can be given to specifiy the hostname used for SNI, which is needed if you
+have multiple SSL hostnames on the same IP address. If not given it will try to
+determine hostname from PeerAddr, which will fail if only IP was given or if
+this argument is used within start_SSL.
+
+If you want to disable SNI set this argument to ''.
+
+Currently only supported for the client side and will be ignored for the server
+side.
+
 =item SSL_version
 
-Sets the version of the SSL protocol used to transmit data.	 The default is SSLv2/3,
+Sets the version of the SSL protocol used to transmit data. The default is 'SSLv23',
 which auto-negotiates between SSLv2 and SSLv3.	You may specify 'SSLv2', 'SSLv3', or
 'TLSv1' (case-insensitive) if you do not want this behavior.
 
+You can limit to set of supported protocols by adding !version separated by ':'.
+The default is 'SSLv23:!SSLv2' which means, that SSLv2, SSLv3 and TLSv1 are supported
+for initial protocol handshakes, but SSLv2 will not be accepted, leaving only
+SSLv3 and TLSv1.  
+
+Setting the version instead to 'TLSv1' will probably break interaction with lots of
+clients which start with SSLv2 and then upgrade to TLSv1.
+
 =item SSL_cipher_list
 
 If this option is set the cipher list for the connection will be set to the
 given value, e.g. something like 'ALL:!LOW:!EXP:!ADH'. Look into the OpenSSL
 documentation (L<http://www.openssl.org/docs/apps/ciphers.html#CIPHER_STRINGS>)
 for more details.
-If this option is not used the openssl builtin default is used which is suitable
-for most cases.
+
+If this option is not set 'ALL:!LOW' will be used.
+To use OpenSSL builtin default (whatever this is) set it to ''.
+
+=item SSL_honor_cipher_order
+
+If this option is true the cipher order the server specified is used instead
+of the order proposed by the client. To mitigate BEAST attack you might use
+something like
+
+  SSL_honor_cipher_order => 1,
+  SSL_cipher_list => 'RC4-SHA:ALL:!ADH:!LOW',
 
 =item SSL_use_cert
 
@@ -1785,13 +1959,17 @@ See the OpenSSL documentation for SSL_CTX_set_verify for more information.
 
 Set the scheme used to automatically verify the hostname of the peer.
 See the information about the verification schemes in B<verify_hostname>.
+
 The default is undef, e.g. to not automatically verify the hostname.
+If no verification is done the other B<SSL_verifycn_*> options have
+no effect, but you might still do manual verification by calling
+B<verify_hostname>.
 
 =item SSL_verifycn_name
 
 Set the name which is used in verification of hostname. If SSL_verifycn_scheme
 is set and no SSL_verifycn_name is given it will try to use the PeerHost and
-PeerAddr settings and fail if no name caan be determined.
+PeerAddr settings and fail if no name can be determined.
 
 Using PeerHost or PeerAddr works only if you create the connection directly
 with C<< IO::Socket::SSL->new >>, if an IO::Socket::INET object is upgraded
@@ -1850,6 +2028,9 @@ The session cache size refers to the number of unique host/port pairs that can b
 stored at one time; the oldest sessions in the cache will be removed if new ones are
 added.
 
+This option does not effect the session cache a server has for it's clients, e.g. it
+does not affect SSL objects with SSL_server set.
+
 =item SSL_session_cache
 
 Specifies session cache object which should be used instead of creating a new.
@@ -1862,6 +2043,12 @@ C<< IO::Socket::SSL::Session_Cache->new( cachesize ) >>.
 
 Use set_default_session_cache() to set a global cache object.
 
+=item SSL_session_id_context
+
+This gives an id for the servers session cache. It's necessary if you want
+clients to connect with a client certificate. If not given but SSL_verify_mode
+specifies the need for client certificate a context unique id will be picked.
+
 =item SSL_error_trap
 
 When using the accept() or connect() methods, it may be the case that the
@@ -1872,6 +2059,16 @@ be closed forcibly.	 The subroutine, if called, will be passed two parameters:
 a reference to the socket on which the SSL negotiation failed and and the full
 text of the error message.
 
+=item SSL_npn_protocols
+
+If used on the server side it specifies list of protocols advertised by SSL
+server as an array ref, e.g. ['spdy/2','http1.1']. 
+On the client side it specifies the protocols offered by the client for NPN
+as an array ref.
+See also method L<next_proto_negotiated>.
+
+Next Protocol Negotioation (NPN) is available with Net::SSLeay 1.46+ and openssl-1.0.1+.
+
 =back
 
 =item B<close(...)>
@@ -1994,11 +2191,15 @@ name will be only checked if no names are given in subjectAltNames.
 This RFC doesn't say much useful about the verification so it just assumes
 that subjectAltNames are possible, but no wildcards are possible anywhere.
 
+=item none
+
+No verification will be done.
+Actually is does not make any sense to call verify_hostname in this case.
+
 =back
 
 The scheme can be given either by specifying the name for one of the above predefined
-schemes, by using a callback (see below) or by using a hash which can have the
-following keys and values:
+schemes, or by using a hash which can have the following keys and values:
 
 =over 8
 
@@ -2020,13 +2221,24 @@ or even '*' will not be allowed.
 Similar to wildcards_in_alt, but checks the common name. There is no predefined
 scheme which allows wildcards in common names.
 
-=back
+=item callback: \&coderef
 
 If you give a subroutine for verification it will be called with the arguments
 ($hostname,$commonName,@subjectAltNames), where hostname is the name given for
 verification, commonName is the result from peer_certificate('cn') and
 subjectAltNames is the result from peer_certificate('subjectAltNames').
 
+All other arguments for the verification scheme will be ignored in this case.
+
+=back
+
+=item B<next_proto_negotiated()>
+
+This method returns the name of negotiated protocol - e.g. 'http/1.1'. It works
+for both client and server side of SSL connection.
+
+NPN support is available with Net::SSLeay 1.46+ and openssl-1.0.1+.
+
 =item B<errstr()>
 
 Returns the last error (in string form) that occurred.	If you do not have a real
@@ -2206,12 +2418,12 @@ See the 'example' directory.
 
 =head1 BUGS
 
-IO::Socket::SSL is not threadsafe.
-This is because IO::Socket::SSL is based on Net::SSLeay which
-uses a global object to access some of the API of openssl
-and is therefore not threadsafe.
-It might probably work if you don't use SSL_verify_callback and
-SSL_password_cb.
+IO::Socket::SSL depends on Net::SSLeay.  Up to version 1.43 of Net::SSLeay
+it was not thread safe, although it did probably work if you did not use 
+SSL_verify_callback and SSL_password_cb.
+
+Creating an IO::Socket::SSL object in one thread and closing it in another
+thread will not work.
 
 IO::Socket::SSL does not work together with Storable::fd_retrieve/fd_store.
 See BUGS file for more information and how to work around the problem.
